/*
  Argot Blink
  
  A very simple example that shows a data packet being delivered over serial
  to command the delayOn/delayOff times of the blink demo.
  
  More information at www.argot-sdk.org/examples/arduino-blink.htm
 
  This example code is in the public domain.
 */
 
// Pin 13 has an LED connected on most Arduino boards.
// give it a name:
int led = 13;
int baudrate = 9600;

int delayOn = 1000;
int delayOff = 1000;

// the setup routine runs once when you press reset:
void setup() {                

  Serial.begin(baudrate);  
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }
  
  // prints title with ending line break 
  Serial.println("Arduino: Argot Blink service started."); 
  
    // initialize the digital pin as an output.
  pinMode(led, OUTPUT);  
}

// the loop routine runs over and over again forever:
void loop() 
{
  digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
  delay(delayOn);            // wait for a second
 
  digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
  delay(delayOff);           // wait for a second
 
}


boolean gotHeader = false;
byte message[32];
byte messageLength = 0;
byte commandLength = 0;
byte read_pos = 0;


// Read a 16bit unsigned integer from the message buffer.
uint16_t read_u16()
{
	uint16_t u16 = 0;
        byte *p = (byte *)&u16;
	p[0] = message[read_pos+1];
        p[1] = message[read_pos];
	read_pos += 2;
	return u16;
}

// Read an 8bit unsigned integer from the message buffer.
uint8_t read_u8()
{
	return message[read_pos++];
}

// Process the command after receiving the complete packet.
void processCommand()
{
  Serial.println("Arduino: Process command message");
  uint8_t command = read_u8();
  uint16_t delay = read_u16();

  if (command == 0)
  {
    Serial.print("Arduino: Received command set delay on: ");
    Serial.println(delay);
    delayOn = delay;
  }
  else
  {
    Serial.print("Arduino: Received command set delay off: ");
    Serial.println(delay);
    delayOff = delay;
  }
}

// Receive the data over serial.  Waits for full message before processing.
void serialEvent() {
  while (Serial.available()) 
  {
    // get the new byte:
    char inChar = (char)Serial.read(); 
    // add it to the inputString:
  
    if (gotHeader == false)
    {
      if (inChar == 'a')
      {
        Serial.println("Arduino: got header character");
        gotHeader = true;
	continue;
      }
    }
    else if (gotHeader == true && commandLength == 0)
    {
      commandLength = inChar;
      messageLength = 0;
      Serial.print("Arduino: got command length: ");
      Serial.println(commandLength);
    }
    else
    {
      message[messageLength] = inChar;
      commandLength--;
      messageLength++;
      
      if (commandLength==0)
      {
        gotHeader = false;
        read_pos = 0;
        processCommand();
      }
    }
  }
}
