/*
 *  
 * 
 * 
 *  This example code is in the public domain.
 */


package com.argot.blink;

import java.io.IOException;
import java.io.InputStream;

import com.argot.TypeException;
import com.argot.TypeLibrary;
import com.argot.TypeMap;
import com.argot.TypeMapperCore;
import com.argot.TypeMapperDynamic;
import com.argot.TypeMapperError;
import com.argot.TypeOutputStream;
import com.argot.blink.Packet.Command;
import com.argot.compiler.ArgotCompilerLoader;
import com.argot.network.TypeEndPoint;
import com.argot.remote.command.RemoteException;
import com.argot.remote.serial.SerialTransport;

public class StartHere 
{
	private TypeLibrary typeLibrary;
	
	/** Milliseconds to block while waiting for port open */
	private static final int TIME_OUT = 2000;
	/** Default bits per second for COM port. */
	private static final int DATA_RATE = 9600;

	private static final String PORT_NAMES[] = 
	{ 
			"/dev/tty.usbserial-A700eh6C", // Mac OS X
			"/dev/ttyUSB0", // Linux
			"COM3", // Windows
	};
	
	
	public static void main(String[] args) 
	{
		try 
		{
			StartHere main = new StartHere();
			main.blink();
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		} 
	}
	

	public StartHere()
	throws TypeException
	{
		// Create the type library and compile/bind the switch data types.
		typeLibrary = new TypeLibrary( );
		ArgotCompilerLoader loader = new ArgotCompilerLoader("blink.argot")
		{
			public void bind( TypeLibrary library )
			throws TypeException
			{
				library.bind( library.getTypeId(Packet.TYPENAME, "1.0"), Packet.class );
			}
		};
		typeLibrary.loadLibrary(loader);
	}
	
	/*
	 * Demonstration of opening the serial port and sending a couple of simple
	 * command packets to the Arduino.
	 */
	
	public void blink() 
	throws TypeException, IOException, RemoteException, InterruptedException
	{
		// Open serial link.
		SerialTransport transport = new SerialTransport(PORT_NAMES, DATA_RATE, TIME_OUT);
		TypeEndPoint endpoint = transport.openLink();
		
		try
		{
			// Creating the reading thread to write messages from Arduino to console.
			SerialRead read = new SerialRead(endpoint.getInputStream());
			Thread readThread = new Thread(read);
			readThread.setDaemon(true);
			readThread.start();
			
			// Prepare the stream.
			TypeMap typeMap = new TypeMap( typeLibrary, new TypeMapperDynamic(new TypeMapperCore(new TypeMapperError())));
			TypeOutputStream typeOut = new TypeOutputStream( endpoint.getOutputStream(), typeMap );
			
			System.out.println("Host: Sleeping 4 seconds");
			Thread.sleep(4000);
			
			// Write the object.
			System.out.println("Host: Writing command 'set delay off 50'");
			Packet command = new Packet(Command.SET_DELAY_OFF,50);		
			typeOut.writeObject( Packet.TYPENAME, command );
	
			System.out.println("Host: Sleeping 10 seconds");
			Thread.sleep(10000);
			
			System.out.println("Host: Writing command 'set delay on 1000'");
			command = new Packet(Command.SET_DELAY_ON,1000);
			typeOut.writeObject( Packet.TYPENAME, command );
	
			System.out.println("Host: Sleeping 10 seconds");
			Thread.sleep(10000);
			
		}
		finally
		{
			System.out.println("Host: Closing link");
			transport.closeLink(endpoint);
		}
	}
	
	/*
	 * A simple Serial reader class that prints everything it receives onto the console.  
	 */
	private static class SerialRead
	implements Runnable
	{
		private InputStream in;
		
		public SerialRead(InputStream in)
		{
			this.in = in;
		}
		@Override
		public void run() 
		{
			while(true)
			{
				char c;
				try 
				{
					c = (char) in.read();
					System.out.print(c);
				} 
				catch (IOException e) 
				{
					e.printStackTrace();
				}
				
			}
		}
		
	}
	
}
